<?php

namespace Drupal\search_api_meilisearch\Api;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\search_api_meilisearch\Client\MeiliSearchClientFactoryInterface;
use MeiliSearch\Client;
use MeiliSearch\Contracts\DocumentsQuery;
use MeiliSearch\Contracts\DocumentsResults;
use MeiliSearch\Contracts\IndexesResults;
use MeiliSearch\Contracts\KeysResults;
use MeiliSearch\Endpoints\Indexes;

/**
 * Provides a service to interact with MeiliSearch API.
 */
class MeiliSearchApiService implements MeiliSearchApiServiceInterface {

  use StringTranslationTrait;

  /**
   * The master key of the server.
   *
   * @var string
   */
  protected string $masterKey = '';

  /**
   * The URL for accessing the meilisearch server.
   *
   * @var string
   */
  protected string $url = '';

  /**
   * The ConfigFactory instance.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The MeiliSearch client.
   *
   * @var \MeiliSearch\Client|null
   */
  protected ?Client $client = NULL;

  /**
   * The MeiliSearch Client factory.
   *
   * @var \Drupal\search_api_meilisearch\Client\MeiliSearchClientFactoryInterface
   */
  protected MeiliSearchClientFactoryInterface $clientFactory;

  /**
   * MeiliSearchApiService constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\search_api_meilisearch\Client\MeiliSearchClientFactoryInterface $client_factory
   *   The MeiliSearch client factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MeiliSearchClientFactoryInterface $client_factory) {
    $this->configFactory = $config_factory;
    $this->clientFactory = $client_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getMasterKey(): string {
    return $this->masterKey;
  }

  /**
   * {@inheritdoc}
   */
  public function setMasterKey(string $key) {
    $this->masterKey = $key;
  }

  /**
   * {@inheritdoc}
   */
  public function getUrl(): string {
    return $this->url;
  }

  /**
   * {@inheritdoc}
   */
  public function setUrl(string $url) {
    $this->url = $url;
  }

  /**
   * {@inheritdoc}
   */
  public function connection(): Client {
    if ($this->client == NULL) {
      $this->client = $this->clientFactory->getInstance($this->getUrl(), $this->getMasterKey());
      try {
        $this->client->version();
      }
      catch (\Exception $e) {
        throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
      }
    }
    return $this->client;
  }

  /**
   * {@inheritdoc}
   */
  public function search(string $indexName, string $query, array $options = []) {
    try {
      $index = $this->getIndex($indexName);
      return $index->search($query, $options);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getIndex(string $indexName): Indexes {
    try {
      return $this->connection()->getIndex($indexName);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createIndex($attributes): array {
    try {
      return $this->connection()->createIndex($attributes, ['primaryKey' => 'search_api_document_id']);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function removeIndex(string $indexName): bool {
    try {
      $task = $this->connection()->deleteIndex($indexName);
      $task = $this->connection()->waitForTask($task['taskUid']);
      return $task['status'] !== 'failed';
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function listIndexes(): IndexesResults {
    try {
      return $this->connection()->getAllIndexes();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function addDocuments(string $indexName, array $documents = []): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->addDocuments($documents, 'search_api_document_id');
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDocument(string $indexName, string $id): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->getDocument($id);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDocuments(string $indexName, DocumentsQuery $options = NULL): DocumentsResults {
    try {
      $index = $this->getIndex($indexName);
      return $index->getDocuments($options);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDocument(string $indexName, string $id): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->deleteDocument($id);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDocuments(string $indexName, array $ids): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->deleteDocuments($ids);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAllDocuments(string $indexName): array {
    try {
      return $this->getIndex($indexName)->deleteAllDocuments();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setFilterableAttributes(string $indexName, array $fields): array {
    try {
      return $this->getIndex($indexName)->updateFilterableAttributes($fields);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getFilterableAttributes(string $indexName): array {
    try {
      return $this->getIndex($indexName)->getFilterableAttributes();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setSettings(string $indexName, array $settings): array {
    try {
      return $this->getIndex($indexName)->updateSettings($settings);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings(string $indexName): array {
    try {
      return $this->getIndex($indexName)->getSettings();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function waitForUpdate(int $updateId): array {
    try {
      return $this->connection()->waitForTask($updateId);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSynonyms(string $indexName): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->getSynonyms();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function updateSynonyms(string $indexName, array $synonyms): array {
    try {
      $index = $this->getIndex($indexName);
      $this->resetSynonyms($indexName);
      return $index->updateSynonyms($synonyms);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function resetSynonyms(string $indexName): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->resetSynonyms();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function updateStopWords(string $indexName, array $stopwords): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->updateStopWords($stopwords);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function resetStopWords(string $indexName): array {
    try {
      $index = $this->getIndex($indexName);
      return $index->resetStopWords();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getRankingRules(string $index_name): array {
    try {
      $index = $this->getIndex($index_name);
      return $index->getRankingRules();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setRankingRules(string $index_name, array $rankingRules): array {
    try {
      $index = $this->getIndex($index_name);
      return $index->updateRankingRules($rankingRules);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSortableAttributes(string $index_name): array {
    try {
      $index = $this->getIndex($index_name);
      return $index->getSortableAttributes();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setSortableAttributes(string $index_name, array $sortableAttributes): array {
    try {
      $index = $this->getIndex($index_name);
      return $index->updateSortableAttributes($sortableAttributes);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSearchableAttributes(string $indexName): array {
    try {
      return $this->getIndex($indexName)->getSearchableAttributes();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setSearchableAttributes(string $indexName, array $searchableAttributes): array {
    try {
      return $this->getIndex($indexName)->updateSearchableAttributes($searchableAttributes);
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function ping(): bool {
    try {
      $this->connection();
      return TRUE;
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function keys(): KeysResults {
    try {
      return $this->connection()->getKeys();
    }
    catch (\Exception $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

}
