<?php

namespace Drupal\search_api_meilisearch\Plugin\search_api\backend;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\search_api\Backend\BackendPluginBase;
use Drupal\search_api\Contrib\AutocompleteBackendInterface;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Item\FieldInterface;
use Drupal\search_api\Item\ItemInterface;
use Drupal\search_api\Plugin\PluginFormTrait;
use Drupal\search_api\Query\QueryInterface;
use Drupal\search_api\SearchApiException;
use Drupal\search_api_autocomplete\SearchInterface;
use Drupal\search_api_autocomplete\Suggestion\SuggestionFactory;
use Drupal\search_api_meilisearch\Api\MeiliSearchApiException;
use Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface;
use Drupal\search_api_meilisearch\Backend\SpecialFieldsInterface;
use Drupal\search_api_meilisearch\Converter\Item\ItemConverterInterface;
use Drupal\search_api_meilisearch\Parser\FilterParserInterface;
use Drupal\search_api_meilisearch\Utility\MeilisearchUtils;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a backend plugin for Search API using MeiliSearch.
 *
 * @SearchApiBackend(
 *   id = "search_api_meilisearch",
 *   label = @Translation("MeiliSearch"),
 *   description = @Translation("Index items using MeiliSearch Server.")
 * )
 */
class SearchApiMeiliSearchBackend extends BackendPluginBase implements AutocompleteBackendInterface, PluginFormInterface, SpecialFieldsInterface {

  use PluginFormTrait;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * MeiliSearch service for making API calls.
   *
   * @var \Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface
   */
  protected MeiliSearchApiServiceInterface $meiliService;

  /**
   * The logger to use for logging messages.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The messenger instance..
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * The filter parser service.
   *
   * @var \Drupal\search_api_meilisearch\Parser\FilterParserInterface
   */
  protected FilterParserInterface $filterParser;

  /**
   * The item list converter service.
   *
   * @var \Drupal\search_api_meilisearch\Converter\Item\ItemConverterInterface
   */
  protected ItemConverterInterface $itemConverter;

  /**
   * The list of warnings.
   *
   * @var array
   */
  protected array $warnings = [];

  /**
   * The construct method for the MeiliSearch back-end plugin.
   *
   * @param array $configuration
   *   Configuration array.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   A plugin definition.
   * @param \Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface $meiliService
   *   A MeiliSearchApiService instance.
   * @param \Psr\Log\LoggerInterface $logger
   *   A LoggerInterface instance.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   A LanguageManagerInterface instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   A ConfigFactory instance.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   A messenger instance.
   * @param \Drupal\search_api_meilisearch\Parser\FilterParserInterface $filterParser
   *   The filter parser service.
   * @param \Drupal\search_api_meilisearch\Converter\Item\ItemConverterInterface $itemConverter
   *   The item list converter service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    MeiliSearchApiServiceInterface $meiliService,
    LoggerInterface $logger,
    LanguageManagerInterface $languageManager,
    ConfigFactoryInterface $configFactory,
    Messenger $messenger,
    FilterParserInterface $filterParser,
    ItemConverterInterface $itemConverter
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = $logger;
    $this->languageManager = $languageManager;
    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->meiliService = $meiliService;
    $this->meiliService->setUrl($this->configuration['meilisearch_host_address'] . ':' . $this->configuration['meilisearch_host_port']);
    $this->meiliService->setMasterKey($this->configuration['meilisearch_master_key']);
    $this->filterParser = $filterParser;
    $this->itemConverter = $itemConverter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('search_api_meilisearch.api'),
      $container->get('logger.channel.search_api_meilisearch'),
      $container->get('language_manager'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('search_api_meilisearch.filter_parser'),
      $container->get('search_api_meilisearch.item_converter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewSettings(): array {
    $indexesList = [];
    try {
      // Available indexes.
      $indexes = $this->meiliService->listIndexes();
      foreach ($indexes as $index) {
        $indexesList[] = $index->getUid();
      }
      $info[] = [
        'label' => $this->t('Available MeiliSearch indexes'),
        'info' => !empty(implode(', ', $indexesList)) ? implode(', ', $indexesList) : $this->t('No indexes found.'),
      ];
      return $info;
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'meilisearch_host_address' => 'http://127.0.0.1',
      'meilisearch_host_port' => 7700,
      'meilisearch_master_key' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {

    $form['meilisearch_host_address'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Host Address:'),
      '#maxlength' => 255,
      '#size' => 30,
      '#required' => TRUE,
      '#description' => $this->t('The host address of the MeiliSearch server.'),
      '#default_value' => $this->configuration['meilisearch_host_address'],
      '#attributes' => [
        'placeholder' => 'http://127.0.0.1',
      ],
    ];

    $form['meilisearch_host_port'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Host Port:'),
      '#maxlength' => 5,
      '#size' => 5,
      '#required' => TRUE,
      '#description' => $this->t('The port of the MeiliSearch server.'),
      '#attributes' => [
        'placeholder' => '7700',
      ],
      '#default_value' => $this->configuration['meilisearch_host_port'],
    ];

    $form['meilisearch_master_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Master Key:'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $this->configuration['meilisearch_master_key'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $hostAddress = $form_state->getValue('meilisearch_host_address');
    if (substr($hostAddress, -1) == '/') {
      $form_state->setValue('meilisearch_host_address', mb_substr($hostAddress, 0, -1));
    }

    $hostPort = (int) $form_state->getValue('meilisearch_host_port');

    if ($hostPort > 65535 || $hostPort <= 0) {
      $form_state->setErrorByName('meilisearch_host_port', $this->t('Host port must be between 1 and 65535'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function removeIndex($index) {
    if ($index->isReadOnly()) {
      return;
    }

    try {
      $this->meiliService->removeIndex($index->id());
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->error($e->getMessage());
      $this->messenger()->addError($this->t('Could not remove @name index.', ['@name' => $index->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function indexItems(IndexInterface $index, array $items): array {
    $documents = $this->itemConverter->convertToDocuments($items);
    try {
      $addDocuments = $this->meiliService->addDocuments($index->id(), $documents);
      $task = $this->meiliService->waitForUpdate($addDocuments['taskUid']);
      if ($task['status'] === 'failed') {
        throw new MeiliSearchApiException($task['error']['message']);
      }
      $this->manageStopWords($index);
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
    }

    return array_keys($items);
  }

  /**
   * {@inheritdoc}
   */
  public function addIndex(IndexInterface $index) {
    try {
      $createIndex = $this->meiliService->createIndex($index->id());
      $task = $this->meiliService->waitForUpdate($createIndex['taskUid']);
      if ($task['status'] === 'failed') {
        throw new MeiliSearchApiException($task['error']['message']);
      }
      $this->updateIndex($index);
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function updateIndex(IndexInterface $index) {
    try {
      $rankingRules = $this->generateRankingRules();
      $oldRankingRules = $this->meiliService->getRankingRules($index->id());
      if ($rankingRules !== $oldRankingRules) {
        $status = $this->meiliService->setRankingRules($index->id(), $rankingRules);
        if (isset($status['taskUid'])) {
          $task = $this->meiliService->waitForUpdate($status['taskUid']);
          if ($task['status'] === 'failed') {
            throw new MeiliSearchApiException($task['error']['message']);
          }
        }
      }

      $searchableAttributes = $this->generateSearchableAttributes($index);
      $oldSearchableAttributes = $this->meiliService->getSearchableAttributes($index->id());
      if ($oldSearchableAttributes != $searchableAttributes) {
        $status = $this->meiliService->setSearchableAttributes($index->id(), $searchableAttributes);
        if (isset($status['taskUid'])) {
          $task = $this->meiliService->waitForUpdate($status['taskUid']);
          if ($task['status'] === 'failed') {
            throw new MeiliSearchApiException($task['error']['message']);
          }
        }
      }

      $sortableAttributes = $this->generateSortableAttributes($index);
      $oldSortableAttributes = $this->meiliService->getSortableAttributes($index->id());
      if ($oldSortableAttributes !== $sortableAttributes) {
        $status = $this->meiliService->setSortableAttributes($index->id(), $sortableAttributes);
        if (isset($status['taskUid'])) {
          $task = $this->meiliService->waitForUpdate($status['taskUid']);
          if ($task['status'] === 'failed') {
            throw new MeiliSearchApiException($task['error']['message']);
          }
        }
      }

      $meiliAttributes = $this->meiliService->getIndex($index->id())->getFilterableAttributes();
      $indexAttributes = array_keys($index->get('field_settings'));
      if (count($meiliAttributes) !== count($indexAttributes)) {
        $filterableAttributes = $index->get('field_settings');
        $facets = array_keys($filterableAttributes);
        $status = $this->meiliService->setFilterableAttributes($index->id(), $facets);
        if (isset($status['taskUid'])) {
          $task = $this->meiliService->waitForUpdate($status['taskUid']);
          if ($task['status'] === 'failed') {
            throw new MeiliSearchApiException($task['error']['message']);
          }
        }
      }

      if ($this->synonymsDisabled($index)) {
        $status = $this->meiliService->resetSynonyms($index->id());
        if (isset($status['taskUid'])) {
          $task = $this->meiliService->waitForUpdate($status['taskUid']);
          if ($task['status'] === 'failed') {
            throw new MeiliSearchApiException($task['error']['message']);
          }
        }
      }
    }
    catch (\Exception $e) {
      $this->handleExceptions($e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getItemSpecialFields(ItemInterface $item): array {
    return $this->getSpecialFields($item->getIndex(), $item);
  }

  /**
   * {@inheritdoc}
   */
  protected function getSpecialFields(IndexInterface $index, ItemInterface $item = NULL): array {
    $fields = parent::getSpecialFields($index, $item);
    $fields['search_api_document_id'] = $this->getFieldsHelper()->createField($index, 'search_api_document_id', [
      'type' => 'string',
      'original type' => 'string',
    ]);

    if ($item) {
      $fields['search_api_document_id']->setValues([MeilisearchUtils::formatAsDocumentId($item->getId())]);
    }

    return $fields;
  }

  /**
   * Check if synonyms processor was disabled.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The current index instance.
   *
   * @return bool
   *   Returns true if synonyms processor was disabled.
   */
  private function synonymsDisabled(IndexInterface $index): bool {
    if (!isset($index->original)) {
      return FALSE;
    }

    $synonymsProcessorEnabled = in_array(
      'search_api_meilisearch_synonyms',
      array_keys($index->getProcessors())
    );
    $synonymsProcessorEnabledOld = in_array(
      'search_api_meilisearch_synonyms',
      array_keys($index->original->getProcessors())
    );

    return !$synonymsProcessorEnabled && $synonymsProcessorEnabledOld;
  }

  /**
   * Generate the Meilisearch ranking rules for the specified search api index.
   *
   * @return string[]
   *   The ranking rules.
   */
  protected function generateRankingRules(): array {
    // We place sort on top, as we want to match by sort first, if a sort is
    // present in the query.
    // @see https://docs.meilisearch.com/learn/core_concepts/relevancy.html#ranking-rules
    return [
      'sort',
      'words',
      'attribute',
      'typo',
      'proximity',
      'exactness',
    ];
  }

  /**
   * Generates the Meilisearch sortable attributes for the index.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The search api index.
   *
   * @return array
   *   The sortable attributes.
   */
  protected function generateSortableAttributes(IndexInterface $index): array {
    $fields = $index->getFields();

    // We allow all fields to be sorted by.
    $sortableAttributes = [];
    foreach ($fields as $field) {
      $sortableAttributes[] = $field->getFieldIdentifier();
    }

    return $sortableAttributes;
  }

  /**
   * Generates the Meilisearch searchable attributes for the index.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The search api index.
   *
   * @return string[]
   *   The searchable attributes.
   */
  protected function generateSearchableAttributes(IndexInterface $index): array {
    $fields = $index->getFields();

    // Sort the index's fields by boost -> a higher boost setting should appear
    // higher in the ranking rules.
    usort($fields, function (FieldInterface $a, FieldInterface $b) {
      if ($a->getBoost() === $b->getBoost()) {
        return 0;
      }

      return ($a->getBoost() < $b->getBoost()) ? 1 : -1;
    });

    $searchableFields = [];
    foreach ($fields as $field) {
      // Only fulltext fields are searchable and have boost settings.
      if ($field->getType() === 'text') {
        $searchableFields[] = $field->getFieldIdentifier();
      }
    }

    return $searchableFields;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteItems(IndexInterface $index, array $item_ids) {
    try {
      $ids = [];
      foreach ($item_ids as $id) {
        $ids[] = MeilisearchUtils::formatAsDocumentId($id);
      }
      $deleteDocuments = $this->meiliService->deleteDocuments($index->id(), $ids);
      $task = $this->meiliService->waitForUpdate($deleteDocuments['taskUid']);
      if ($task['status'] === 'failed') {
        throw new MeiliSearchApiException($task['error']['message']);
      }
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAllIndexItems(IndexInterface $index, $datasource_id = NULL) {
    try {
      $deleteAllDocuments = $this->meiliService->deleteAllDocuments($index->id());
      $task = $this->meiliService->waitForUpdate($deleteAllDocuments['taskUid']);
      if ($task['status'] === 'failed') {
        throw new MeiliSearchApiException($task['error']['message']);
      }
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function search(QueryInterface $query) {
    $results = $query->getResults();
    $options = $query->getOptions();
    $index = $query->getIndex();

    try {
      $filterableAttributes = $index->get('field_settings');
      $facets = array_keys($filterableAttributes);
      $meiliOptions = [];

      if (!empty($facets)) {
        $meiliOptions['facets'] = $facets;
      }

      $filter = $this->filterParser->parseExpression($query->getConditionGroup(), $index);

      // If any facets are set, add them to the search parameters.
      if ($filter) {
        $meiliOptions['filter'] = $filter;
        unset($filter);
      }

      $meiliOptions['limit'] = 0;
      $keys = $query->getOriginalKeys() ?? '';
      $meiliOptions['offset'] = $options['offset'] ?? 0;
      if (isset($options['limit']) && $options['limit'] != 0) {
        $meiliOptions['limit'] = (int) $options['limit'];
      }
      else {
        $emptyQuery = $this->meiliService->search($index->id(), $keys, $meiliOptions);
        $meiliOptions['limit'] = $emptyQuery->getEstimatedTotalHits();
      }

      $resultsOptions = [
        'relevance_ascending' => FALSE,
        'random' => FALSE,
      ];
      $sorts = $query->getSorts();
      foreach ($sorts as $field_name => $order) {
        if ($order != QueryInterface::SORT_ASC && $order != QueryInterface::SORT_DESC) {
          $msg = $this->t('Unknown sort order @order. Assuming "@default".', [
            '@order' => $order,
            '@default' => QueryInterface::SORT_ASC,
          ]);
          $this->warnings[(string) $msg] = 1;
          $order = QueryInterface::SORT_ASC;
        }
        if ($field_name === 'search_api_relevance') {
          if ($order === QueryInterface::SORT_ASC) {
            $resultsOptions['relevance_ascending'] = TRUE;
          }
          // Meilisearch already returns the results by relevance in desc order.
          continue;
        }
        if ($field_name === 'search_api_random') {
          $resultsOptions['random'] = TRUE;
          continue;
        }
        $meiliOptions['sort'][] = $field_name . ':' . strtolower($order);
      }

      $data = $this->meiliService->search($index->id(), $keys, $meiliOptions);
      if (!$data->getHits()) {
        return;
      }
      $results->setResultCount($data->getEstimatedTotalHits());

      if ($data->getFacetDistribution()) {
        $results->setExtraData('search_api_facets', $this->extractFacetsData($data->getFacetDistribution()));
      }

      if (empty($data->getHits())) {
        return;
      }

      $hits = $data->getHits();
      if ($resultsOptions['relevance_ascending'] === TRUE) {
        // We need to reverse the search results.
        $hits = array_reverse($hits, TRUE);
      }
      if ($resultsOptions['random'] === TRUE) {
        shuffle($hits);
      }
      foreach ($hits as $row) {
        if (!isset($row['search_api_id'])) {
          continue;
        }

        $item = $this->getFieldsHelper()->createItem($index, $row['search_api_id']);
        $results->addResultItem($item);
      }
    }
    catch (MeiliSearchApiException $e) {
      $this->handleExceptions($e);
      throw new SearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedFeatures(): array {
    return [
      'search_api_autocomplete',
      'search_api_facets',
      'search_api_facets_operator_or',
      'search_api_random_sort',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getAutocompleteSuggestions(QueryInterface $query, SearchInterface $search, $incomplete_key, $user_input): array {
    $suggestions = [];
    $factory = new SuggestionFactory($user_input);
    $index = $query->getIndex();

    try {
      $firstQuery = $this->meiliService->search($index->id(), $user_input, ['limit' => 0]);
      $allHits = $firstQuery['nbHits'];

      $meiliOptions = [
        'limit' => $allHits,
      ];
      $data = $this->meiliService->search($index->id(), $user_input, $meiliOptions);
    }
    catch (MeiliSearchApiException $e) {
      return $this->handleExceptions($e);
    }

    $entity_ids = [];
    foreach ($data['hits'] as $row) {
      $entity_ids[] = $row['search_api_id'];
    }

    $items = $index->loadItemsMultiple($entity_ids);

    // Sort the autocomplete suggestions based on the user's input.
    // If the suggestion starts with the user's input bubble it to the top.
    foreach ($items as $item) {
      $suggestion = $factory->createFromSuggestedKeys($item->getString());
      if (substr(strtolower($item->getString()), 0, strlen(strtolower($user_input))) === strtolower($user_input)) {
        array_unshift($suggestions, $suggestion);
      }
      else {
        $suggestions[] = $suggestion;
      }
    }

    return array_slice($suggestions, 0, $query->getOption('limit'));
  }

  /**
   * Manages the stopwords list.
   *
   * If the "Stopwords" processor is enabled, add the stopwords entered in
   * the settings to the Meilisearch server. Otherwise, reset the list.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   The current index instance.
   *
   * @return array
   *   An update id or false if everything fails.
   */
  public function manageStopWords(IndexInterface $index): array {
    $processors = $index->getProcessors();
    if (isset($processors['stopwords'])) {
      $indexArray = $index->toArray();
      $stopwords = array_values($indexArray['processor_settings']['stopwords']['stopwords']);
      try {
        return $this->meiliService->updateStopWords($index->id(), $stopwords);
      }
      catch (MeiliSearchApiException $e) {
        return $this->handleExceptions($e);
      }
    }
    else {
      try {
        return $this->meiliService->resetStopWords($index->id());
      }
      catch (MeiliSearchApiException $e) {
        return $this->handleExceptions($e);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isAvailable(): bool {
    try {
      return $this->meiliService->ping() !== FALSE;
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());
    }

    return FALSE;
  }

  /**
   * Handles the exceptions.
   *
   * @param \Exception $exception
   *   The exception that was thrown.
   *
   * @return array
   *   Returns an empty array.
   */
  public function handleExceptions(\Exception $exception): array {
    $this->logger->error($exception->getMessage());
    switch ($exception->getCode()) {
      case(403):
        $this->messenger()
          ->addError($this->t('The master key is not correct.'));
        return [];

      case(0):
        $this->messenger()
          ->addError($this->t('The connection to the server could not be established.'));
        return [];

      default:
        $this->messenger()
          ->addError($exception->getMessage());
        return [];
    }
  }

  /**
   * Extract facets data from response.
   *
   * @param array $data
   *   Facets data from response.
   *
   * @return array
   *   Formatted facets data.
   */
  protected function extractFacetsData(array $data): array {
    $facets = [];
    if (!empty($data)) {
      foreach ($data as $facetName => $terms) {
        $facetData = [];
        foreach ($terms as $name => $count) {
          $facetData['filter'] = "\"$name\"";
          $facetData['count'] = $count;
          $facets[$facetName][] = $facetData;
        }
      }
    }

    return $facets;
  }

  /**
   * {@inheritdoc}
   */
  public function __sleep(): array {
    $properties = array_flip(parent::__sleep());
    unset($properties['meiliService']);

    return array_keys($properties);
  }

  /**
   * {@inheritdoc}
   */
  public function __wakeup(): void {
    parent::__wakeup();

    $container = \Drupal::getContainer();
    $this->meiliService = $container->get('search_api_meilisearch.api');
    $this->meiliService->setUrl($this->configuration['meilisearch_host_address'] . ':' . $this->configuration['meilisearch_host_port']);
    $this->meiliService->setMasterKey($this->configuration['meilisearch_master_key']);
  }

}
