<?php

namespace Drupal\Tests\search_api_meilisearch\Kernel\Update;

use Drupal\Component\Utility\Html;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\search_api\Entity\Index;
use Drupal\search_api\Entity\Server;

/**
 * Provides tests for hook_update_9002.
 *
 * @group search_api_meilisearch
 */
class UpdateHook9002Test extends UpdateHookTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'search_api',
    'search_api_test',
    'search_api_meilisearch',
  ];

  /**
   * {@inheritdoc}
   */
  protected static $configSchemaCheckerExclusions = [
    'search_api_meilisearch.settings',
  ];

  /**
   * The list of various indexes.
   *
   * @var \Drupal\search_api\IndexInterface[]
   */
  protected array $indexes = [];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig('search_api');
    $this->installSchema('search_api', ['search_api_item']);
    $this->installEntitySchema('search_api_task');
    $this->prepareTestIndexes();
    $this->prepareOldConfiguration();
  }

  /**
   * Tests the update hook.
   */
  public function testUpdateHook(): void {
    $this->runUpdateHook(9002);

    // Test that meilisearch indexes are scheduled for reindex.
    $this->assertTrue($this->indexes['index_with_mapped_ids']->isReindexing());
    $this->assertTrue($this->indexes['index_without_mapped_ids']->isReindexing());

    // Test that non-meilisearch index is not scheduled for reindex.
    $this->assertFalse($this->indexes['index_test_backend']->isReindexing());

    // Test that notice message is set.
    $messages = $this->container
      ->get('messenger')
      ->messagesByType(MessengerInterface::TYPE_STATUS);
    $this->assertCount(1, $messages);
    $this->assertEquals(
      'Meilisearch indexes "test_index", "test_index_no_ids" have been scheduled to be reindexed. Until reindexing occurs, searches for the affected indexes will not return any results.',
      Html::decodeEntities($messages[0])
    );

    // Test that configuration was removed from module configuration.
    $config = $this->config('search_api_meilisearch.settings');
    $this->assertArrayNotHasKey('meilisearch_id_mappings', $config->getRawData());

    // Test that the entire configuration has been deleted.
    $this->assertTrue($config->isNew());
  }

  /**
   * Sets up a test index.
   */
  protected function prepareTestIndexes(): void {
    $server = Server::create([
      'id' => 'test_server_1',
      'name' => 'Test server',
      'backend' => 'search_api_meilisearch',
      'backend_config' => [],
    ]);
    $server->save();

    $index = Index::create([
      'id' => 'test_index',
      'name' => 'Test index with mapped ids',
      'server' => $server->id(),
    ]);

    $index->save();
    $this->indexes['index_with_mapped_ids'] = $index;

    $server = Server::create([
      'id' => 'test_server_2',
      'name' => 'Test server',
      'backend' => 'search_api_meilisearch',
      'backend_config' => [],
    ]);
    $server->save();

    $index = Index::create([
      'id' => 'test_index_no_ids',
      'name' => 'Test index without mapped ids',
      'server' => $server->id(),
    ]);

    $index->save();
    $this->indexes['index_without_mapped_ids'] = $index;

    $server = Server::create([
      'id' => 'test_server_3',
      'name' => 'Test server other',
      'backend' => 'search_api_test',
      'backend_config' => [],
    ]);
    $server->save();

    $index = Index::create([
      'id' => 'test_index_search_api_test',
      'name' => 'Test index with test backend plugin',
      'server' => $server->id(),
    ]);

    $index->save();
    $this->indexes['index_test_backend'] = $index;

    // Mark all indexes as not scheduled for reindex.
    $state = $this->container->get('state');
    $key = 'search_api.index.test_index.has_reindexed';
    $state->set($key, FALSE);
    $key = 'search_api.index.test_index_no_ids.has_reindexed';
    $state->set($key, FALSE);
    $key = 'search_api.index.test_index_search_api_test.has_reindexed';
    $state->set($key, FALSE);
  }

  /**
   * Sets the old configuration before updates are run.
   */
  protected function prepareOldConfiguration(): void {
    $this->config('search_api_meilisearch.settings')
      ->set('meilisearch_id_mappings', [
        'test_index' => [
          'entity_node_1_en' => 'entity:node/1:en',
          'entity_node_2_en' => 'entity:node/2:en',
          'entity_node_3_en' => 'entity:node/3:en',
        ],
      ])->save(TRUE);
  }

}
